from .base import HomeServerApp


class HomeAssistant(HomeServerApp):
    @property
    def config_file(self):
        return self.path / 'config' / 'configuration.yaml'

    def setup(self):
        """Setup home assistant base url."""
        self.container.stop()
        self.setup_nginx()
        self.container.start()

    def setup_nginx(self):
        """Setup base url to use it as a proxy server. It requires the FULL url."""
        config = self.config_file.read_text()
        if not 'base_url' in config:
            config += f"http:\n  base_url: https://{self.domain}/homeassistant/"
        self.config_file.write_text(config)



import configparser
from .base import HomeServerApp


class Medusa(HomeServerApp):
    @property
    def config_file(self):
        return self.path / 'config.ini'

    def setup_nginx(self):
        self.container.stop()
        config = configparser.ConfigParser()
        config.read(str(self.config_file))
        config.set('General', 'web_root', '/tv')
        with open(self.config_file, 'w') as fileo:
            config.write(fileo)
        self.compose.start()

    def setup(self):
        self.setup_nginx()



import json
from .base import HomeServerApp


class Jackett(HomeServerApp):
    def setup(self):
        self.container.stop()
        self.setup_nginx()
        self.compose.start()

    def setup_nginx(self):
        config = self.path / 'Jackett' / 'ServerConfig.json'
        result = json.load(config.open('r'))
        result['BasePathOverride'] = '/trackers'
        json.dump(result, config.open('w'))



import json
import uuid
from .base import HomeServerApp


class NextCloud(HomeServerApp):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.password = uuid.uuid4().hex
        self.logger.info(
            f'Setting up admin password as "{self.password}", save it.')

    def setup(self):
        self.logger.debug('Fixing directory permissions permissions')
        self.fix_permissions()
        self.logger.debug('Installing nextcloud')
        self.install()
        self.logger.debug('Creating new user')
        self.create_users()
        self.logger.debug('Setting up nginx paths')
        self.setup_paths()
        self.logger.debug('Installing external links apps')
        self.install_external_links()
        self.logger.debug('Setting up onlyoffice')
        self.setup_onlyoffice()
        self.logger.debug('Setting up music')
        self.setup_music()
        self.logger.debug('Setting up SMS sync')
        self.setup_ocsms()
        self.logger.debug('Setting up tasks')
        self.setup_tasks()
        self.logger.debug('Setting up calendar')
        self.setup_calendar()
        self.logger.debug('Setting up carnet')
        self.setup_carnet()
        self.logger.debug('Setting up keepass web')
        self.setup_keeweb()
        self.logger.debug('Setting up deck')
        self.setup_deck()
        self.logger.debug('Setting up contacts')
        self.setup_contacts()
        self.logger.debug('Set up nextcloud')

    def fix_permissions(self):
        self.container.exec_run(['chown', '-R', 'www-data', '/data'],
                                stdout=False,
                                privileged=True)

    def install(self):
        self.occ('maintenance:install', '--data-dir', '/data', '--admin-pass',
                 self.password, '--admin-user', 'admin')

    def create_users(self):
        self.occ('user:add', '--password-from-env', '--display-name',
                 self.meta['nextcloud_username'],
                 self.meta['nextcloud_username'])

    def setup_paths(self):
        self.occ('config:system:set', 'overwritewebroot', '--value', '/')
        self.occ('config:system:set', 'overwriteprotocol', '--value', 'https')
        self.occ('config:system:set', 'trusted_domains', '0', '--value',
                 self.domain)

    def setup_music(self):
        self.occ('app:install', 'music')

    def setup_ocsms(self):
        self.occ('app:install', 'ocsms')

    def setup_tasks(self):
        self.occ('app:install', 'tasks')

    def setup_calendar(self):
        """Setup calendar"""
        self.occ('app:install', 'calendar')

    def setup_carnet(self):
        """Setup carnet note-taking app"""
        self.occ('app:install', 'carnet')

    def setup_deck(self):
        """Setup deck kanban"""
        self.occ('app:install', 'deck')

    def setup_contacts(self):
        """Setup contacts"""
        self.occ('app:install', 'contacts')

    def setup_keeweb(self):
        self.occ('app:install', 'keeweb')

    def install_external_links(self):
        """Install top links to all the rest of apps, to centralice everything on nextcloud"""
        self.occ('app:install', 'external')
        self.occ(
            'config:app:set', 'external', 'sites', '--value',
            json.dumps({
                "1": {
                    "id": 1,
                    "name": "Series downloads",
                    "url": f"https://{self.domain}/tv/",
                    "lang": "",
                    "type": "link",
                    "device": "",
                    "icon": "external.svg",
                    "groups": [],
                    "redirect": False
                },
                "2": {
                    "id": 2,
                    "name": "Movies downloads",
                    "url": f"https://{self.domain}/movies/",
                    "lang": "",
                    "type": "link",
                    "device": "",
                    "icon": "external.svg",
                    "groups": [],
                    "redirect": False
                },
                "3": {
                    "id": 3,
                    "name": "Music downloads",
                    "url": f"https://{self.domain}/music/",
                    "lang": "",
                    "type": "link",
                    "device": "",
                    "icon": "external.svg",
                    "groups": [],
                    "redirect": False
                },
                "4": {
                    "id": 4,
                    "name": "Book downloads",
                    "url": f"https://{self.domain}/books/",
                    "lang": "",
                    "type": "link",
                    "device": "",
                    "icon": "external.svg",
                    "groups": [],
                    "redirect": False
                },
                "5": {
                    "id": 5,
                    "name": "Media player",
                    "url": f"https://{self.domain}/jellyfin/",
                    "lang": "",
                    "type": "link",
                    "device": "",
                    "icon": "external.svg",
                    "groups": [],
                    "redirect": False
                }
            }))

    def setup_onlyoffice(self):
        trusted_domains = self.occ('config:system:get',
                                   'trusted_domains').splitlines()

        if not any([b'nginx' in a for a in trusted_domains]):
            self.logger.debug("Nginx not found in trusted domains, setting up")
            self.occ('config:system:set', 'trusted_domains',
                     str(len(trusted_domains)), '--value', 'nginx')

        self.logger.debug("Installing onlyoffice app")
        self.occ('app:install', 'onlyoffice')
        self.occ('config:system:set', 'onlyoffice', 'DocumentServerUrl',
                 '--value', '/ds-vpath/')
        self.occ('config:system:set', 'onlyoffice',
                 'DocumentServerInternalUrl', '--value',
                 'http://onlyoffice-document-server/')
        self.containers['onlyoffice-document-server'].exec_run([
            "sed", "-i",
            's/"rejectUnauthorized": true/"rejectUnauthroized": false/g',
            '/etc/onlyoffice/documentserver/default.json'
        ])
        self.occ('config:system:set', 'onlyoffice', 'StorageUrl', '--value',
                 'https://nginx/"')

    def occ(self, *args, **kwargs):
        try:
            args = ['php', 'occ', '--no-warnings', *args]
            kwargs.update(
                dict(user='www-data',
                     stdout=True,
                     demux=False,
                     stderr=False,
                     environment={'OC_PASS': self.meta['nextcloud_password']}))
            result = self.container.exec_run(args, **kwargs)
            self.logger.debug(result)
            return result[1]
        except:
            self.logger.exception('could not execute command')



from pathlib import Path
from bacchus.base import HomeServerApp


class OpenVPN(HomeServerApp):
    def setup(self):
        try:
            self.logger.debug(
                self.run("ovpn_genconfig", "-u", f"udp://{self.domain}"))
            self.logger.debug(
                self.run('bash', '-c',
                         f'echo {self.domain}|ovpn_initpki nopass'))
            # self.container.stop()
            self.compose.start()
            self.logger.debug(
                self.run('easyrsa', 'build-client-full',
                         self.meta['nextcloud_username'], 'nopass'))
            response = self.run('ovpn_getclient',
                                self.meta['nextcloud_username'])
            Path('vpn_client.config').write_text(response)
        except Exception as err:
            self.logger.exception('could not create openvpn config')

    def run(self, *cmd):
        volumes = {
            self.path.absolute(): {
                'bind': '/etc/openvpn',
                'mode': 'rw'
            }
        }
        return self.client.containers.run('kylemanna/openvpn',
                                          command=cmd,
                                          tty=True,
                                          volumes=volumes,
                                          detach=False)



from .base import HomeServerApp
from .base import TEMPLATES
from pathlib import Path
import subprocess


class Nginx(HomeServerApp):
    def setup_certificates(self):
        """Setup certificates"""
        data_path = self.path / self.domain
        data_path.mkdir(parents=True, exist_ok=True)
        local_path = '/etc/certs/'

        try:
            cmd = [
                "req", "-x509", "-nodes", "-newkey", "rsa:4096", "-days",
                "365", "-keyout", f"{local_path}{self.domain}/privkey.pem",
                "-out", f"{local_path}/{self.domain}/fullchain.pem", "-subj",
                f"/CN={self.domain}"
            ]
            self.logger.debug(f'executing {" ".join(cmd)}')
            result = self.client.containers.run('frapsoft/openssl',
                                                command=cmd,
                                                volumes={
                                                    self.path.absolute(): {
                                                        'bind': local_path,
                                                        'mode': 'rw'
                                                    }
                                                },
                                                auto_remove=True,
                                                detach=False)
            self.logger.debug(result)
        except Exception as err:
            self.logger.exception('could not create ssl certs')
        self.logger.debug(list(data_path.glob('*')))

    def setup_config(self):
        """Write nginx config."""
        (self.path / 'nginx.conf').write_text(
            (TEMPLATES / 'nginx.tpl').read_text().format(domain=self.domain))

    def setup(self):
        self.logger.debug("Setting up certificates")
        self.setup_certificates()
        self.logger.debug('Setting nginx configuration template')
        self.setup_config()



from functools import lru_cache
import xml.etree.ElementTree as ET

from .base import HomeServerApp


class Radarr(HomeServerApp):
    @property
    def config_file(self):
        return self.path / 'config.xml'

    @property
    @lru_cache()
    def config(self):
        return ET.parse(str(self.config_file))

    def setup_nginx(self):
        self.config.find('UrlBase').text = '/movies/'
        self.config.write(str(self.config_file))

    def setup(self):
        self.setup_nginx()
